/* Output from p2c, the Pascal-to-C translator */
/* From input file "bgidemo.pas" */


/* Turbo Graphics */
/* Copyright (c) 1985, 1990 by Borland International, Inc. */

/* Changed in 1993 by Hartmut Schirmer */


#include <p2c/p2c.h>


/*
  Turbo Pascal 6.0 Borland Graphics Interface (BGI) demonstration
  program. This program shows how to use many features of
  the Graph unit.

  NOTE: to have this demo use the IBM8514 driver, specify a
  conditional define constant "Use8514" (using the {$DEFINE}
  directive or Options\Compiler\Conditional defines) and then
  re-compile.

*/

#ifndef CRT_H
#include <gppconio.h>
#endif

#ifndef GRAPH_H
#include <p2c/tp2bcc.h>
#endif


/*, Dos*/


/* The five fonts available */

Static Char Fonts[5][14] = {
  "DefaultFont", "TriplexFont", "SmallFont", "SansSerifFont", "GothicFont"
};

/* The five predefined line styles supported */

Static Char LineStyles[5][10] = {
  "SolidLn", "DottedLn", "CenterLn", "DashedLn", "UserBitLn"
};

/* The twelve predefined fill styles supported */

Static Char FillStyles[12][15] = {
  "EmptyFill", "SolidFill", "LineFill", "LtSlashFill", "SlashFill",
  "BkSlashFill", "LtBkSlashFill", "HatchFill", "XHatchFill", "InterleaveFill",
  "WideDotFill", "CloseDotFill"
};

/* The two text directions available */

Static Char TextDirect[2][9] = {
  "HorizDir", "VertDir"
};

/* The Horizontal text justifications available */

Static Char HorizJust[3][11] = {
  "LeftText", "CenterText", "RightText"
};

/* The vertical text justifications available */

Static Char VertJust[3][11] = {
  "BottomText", "CenterText", "TopText"
};

Static int GraphDriver;   /* The Graphics device driver */
Static int GraphMode;   /* The Graphics mode value */
Static unsigned short MaxX, MaxY;   /* The maximum resolution of the screen */
Static int ErrorCode;   /* Reports any graphics errors */
Static unsigned short MaxColor;   /* The maximum color value available */
Static void *OldExitProc;   /* Saves exit procedure address */


/*$F+
procedure MyExitProc;
begin
  ExitProc := OldExitProc; { Restore exit procedure address }
  CloseGraph;              { Shut down the graphics system }
end; { MyExitProc }
$F-*/

Static void Initialize(void)
{
  /* Initialize graphics and report any errors that may occur */
  boolean InGraphicsMode;   /* Flags initialization of graphics mode */
  Char PathToDriver[256];   /* Stores the DOS path to *.BGI & *.CHR */
  Char STR2[256];

  /* when using Crt and graphics, turn off Crt's memory-mapped writes */
  /*
    DirectVideo := False;
    OldExitProc := ExitProc;                { save previous exit proc }
    ExitProc := @MyExitProc;                { insert our exit proc in chain }
  */
  strcpy(PathToDriver, "..\\chr");
  do {

    /* Use8514
        GraphDriver := IBM8514;
        GraphMode := IBM8514Hi;
    */
    GraphDriver = DETECT;   /* use autodetection */

    initgraph(&GraphDriver, &GraphMode, PathToDriver);
    ErrorCode = graphresult();   /* preserve error return */
    if (ErrorCode != grOk) {   /* error? */
      printf("Graphics error: %s\n", grapherrormsg(ErrorCode));
      if (ErrorCode != grFileNotFound)   /* Can't find driver file */
	_Escape(1);
      /* Some other error: terminate */
      printf("Enter full path to BGI driver or type <Ctrl-Break> to quit:\n");
      gets(PathToDriver);
      putchar('\n');
    }
  } while (ErrorCode != grOk);
  RANDZE_TP();   /* init random number generator */
  MaxColor = getmaxcolor();   /* Get the maximum allowable drawing color */
  MaxX = getmaxx();   /* Get screen resolution values */
  MaxY = getmaxy();
}  /* Initialize */


Static Char *Int2Str(Char *Result, long L)
{
  /* Converts an integer to a string for use with OutText, OutTextXY */
  Char S[256];

  sprintf(S, "%ld", L);
  return strcpy(Result, S);
}  /* Int2Str */


Static unsigned short RandColor(void)
{
  /* Returns a Random non-zero color value that is within the legal
     color range for the selected device driver and graphics mode.
     MaxColor is set to GetMaxColor by Initialize */
  return (RANDOM_TP(MaxColor) + 1);
}  /* RandColor */


Static void DefaultColors(void)
{
  /* Select the maximum color in the Palette for the drawing color */
  setcolor(WHITE);
}  /* DefaultColors */


Static void DrawBorder(void)
{
  /* Draw a border around the current view port */
  ViewPortType ViewPort;

  DefaultColors();
  setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
  getviewsettings(&ViewPort);
  rectangle(0, 0, ViewPort.right - ViewPort.left,
	    ViewPort.bottom - ViewPort.top);
}  /* DrawBorder */


Static void FullPort(void)
{
  /* Set the view port to the entire screen */
  setviewport(0, 0, MaxX, MaxY, ClipOn);
}  /* FullPort */


Static void MainWindow(Char *Header)
{
  /* Make a default window and view port for demos */
  DefaultColors();   /* Reset the colors */
  cleardevice();   /* Clear the screen */
  settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);   /* Default text font */
  settextjustify(CENTER_TEXT, TOP_TEXT);   /* Left justify text */
  FullPort();   /* Full screen view port */
  outtextxy(MaxX / 2, 2, Header);   /* Draw the header */
  /* Draw main window */
  setviewport(0, textheight("M") + 4, MaxX, MaxY - textheight("M") - 4,
	      ClipOn);
  DrawBorder();   /* Put a border around it */
  /* Move the edges in 1 pixel on all sides so border isn't in the view port */
  setviewport(1, textheight("M") + 5, MaxX - 1, MaxY - textheight("M") - 5,
	      ClipOn);
}  /* MainWindow */


Static void StatusLine(Char *Msg)
{
  /* Display a status line at the bottom of the screen */
  FullPort();
  DefaultColors();
  settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);
  settextjustify(CENTER_TEXT, TOP_TEXT);
  setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
  setfillstyle(EMPTY_FILL, 0);
  bar(0, MaxY - textheight("M") - 4, MaxX, MaxY);
      /* Erase old status line */
  rectangle(0, MaxY - textheight("M") - 4, MaxX, MaxY);
  outtextxy(MaxX / 2, MaxY - textheight("M") - 2, Msg);
  /* Go back to the main window */
  setviewport(1, textheight("M") + 5, MaxX - 1, MaxY - textheight("M") - 5,
	      ClipOn);
}  /* StatusLine */


#define Esc             '\033'


Static void WaitToGo(void)
{
  /* Wait for the user to abort the program or continue */
  Char Ch;

  StatusLine("Esc aborts or press a key...");
  do {
  } while (!kbhit());
  Ch = getch();
  if (Ch == '\0')   /* trap function keys */
    Ch = getch();
  if (Ch == Esc) {   /* clear screen, go on with demo */
    restorecrtmode();   /* terminate program */
    _Escape(0);
  }
  cleardevice();
}  /* WaitToGo */

#undef Esc


Static void GetDriverAndMode(Char *DriveStr, Char *ModeStr)
{
  /* Return strings describing the current device driver and graphics mode
     for display of status report */
  Char STR1[256];

  strcpy(DriveStr, getdrivername());
  strcpy(ModeStr, getmodename(getgraphmode()));
}  /* GetDriverAndMode */


#define X               10


/* Local variables for ReportStatus: */
struct LOC_ReportStatus {
  unsigned short Y;
} ;

Local void WriteOut(Char *S, struct LOC_ReportStatus *LINK)
{
  /* Write out a string and increment to next line */
  outtextxy(X, LINK->Y, S);
  LINK->Y += textheight("M") + 2;
}  /* WriteOut */


Static void ReportStatus(void)
{
  /* Display the status of all query functions after InitGraph */
  struct LOC_ReportStatus V;
  ViewPortType ViewInfo;   /* Parameters for inquiry procedures */
  LineSettingsType LineInfo;
  FillSettingsType FillInfo;
  TextSettingsType TextInfo;
  PaletteType Palette;
  Char DriverStr[256];   /* Driver and mode strings */
  Char ModeStr[256];
  Char STR1[256], STR2[256], STR3[256], STR4[256], STR5[256];
  Char STR6[32];
  Char STR7[36];
  Char STR8[30];

  GetDriverAndMode(DriverStr, ModeStr);   /* Get current settings */
  getviewsettings(&ViewInfo);
  getlinesettings(&LineInfo);
  getfillsettings(&FillInfo);
  gettextsettings(&TextInfo);
  getpalette(&Palette);

  V.Y = 4;
  MainWindow("Status report after InitGraph");
  settextjustify(LEFT_TEXT, TOP_TEXT);
  sprintf(STR1, "Graphics device    : %s", DriverStr);
  WriteOut(STR1, &V);
  sprintf(STR1, "Graphics mode      : %s", ModeStr);
  WriteOut(STR1, &V);
  sprintf(STR3, "Screen resolution  : (0, 0, %s, %s)",
	  Int2Str(STR1, getmaxx()), Int2Str(STR2, getmaxy()));
  WriteOut(STR3, &V);
  sprintf(STR3, "Current view port  : (%s, %s, %s, %s)",
	  Int2Str(STR1, ViewInfo.left), Int2Str(STR2, ViewInfo.top),
	  Int2Str(STR4, ViewInfo.right), Int2Str(STR5, ViewInfo.bottom));
  WriteOut(STR3, &V);
  if (ViewInfo.clip)
    WriteOut("Clipping           : ON", &V);
  else
    WriteOut("Clipping           : OFF", &V);
  sprintf(STR4, "Current position   : (%s, %s)",
	  Int2Str(STR1, getx()), Int2Str(STR2, gety()));
  WriteOut(STR4, &V);
  sprintf(STR2, "Palette entries    : %s", Int2Str(STR1, Palette.size));
  WriteOut(STR2, &V);
  sprintf(STR2, "GetMaxColor        : %s", Int2Str(STR1, getmaxcolor()));
  WriteOut(STR2, &V);
  sprintf(STR2, "Current color      : %s", Int2Str(STR1, getcolor()));
  WriteOut(STR2, &V);
  sprintf(STR6, "Line style         : %s", LineStyles[LineInfo.linestyle]);
  WriteOut(STR6, &V);
  sprintf(STR2, "Line thickness     : %s", Int2Str(STR1, LineInfo.thickness));
  WriteOut(STR2, &V);
  sprintf(STR7, "Current fill style : %s", FillStyles[FillInfo.pattern]);
  WriteOut(STR7, &V);
  sprintf(STR2, "Current fill color : %s", Int2Str(STR1, FillInfo.color));
  WriteOut(STR2, &V);
  sprintf(STR7, "Current font       : %s", Fonts[TextInfo.font]);
  WriteOut(STR7, &V);
  sprintf(STR8, "Text direction     : %s", TextDirect[TextInfo.direction]);
  WriteOut(STR8, &V);
  sprintf(STR2, "Character size     : %s", Int2Str(STR1, TextInfo.charsize));
  WriteOut(STR2, &V);
  sprintf(STR6, "Horizontal justify : %s", HorizJust[TextInfo.horiz]);
  WriteOut(STR6, &V);
  sprintf(STR6, "Vertical justify   : %s", VertJust[TextInfo.vert]);
  WriteOut(STR6, &V);
  WaitToGo();
}  /* ReportStatus */

#undef X


#define MaxFillStyles   12   /* patterns 0..11 */


Static void FillEllipsePlay(void)
{
  /* Random filled ellipse demonstration */
  unsigned short MaxRadius;
  int FillColor;

  MainWindow("FillEllipse demonstration");
  StatusLine("Esc aborts or press a key");
  MaxRadius = MaxY / 10;
  setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
  do {
    FillColor = RandColor();
    setcolor(FillColor);
    setfillstyle(RANDOM_TP(MaxFillStyles), FillColor);
    fillellipse(RANDOM_TP(MaxX), RANDOM_TP(MaxY), RANDOM_TP(MaxRadius),
		RANDOM_TP(MaxRadius));
  } while (!kbhit());
  WaitToGo();
}  /* FillEllipsePlay */

#undef MaxFillStyles


#define MaxFillStyles   12   /* patterns 0..11 */


Static void SectorPlay(void)
{
  /* Draw random sectors on the screen */
  unsigned short MaxRadius;
  int FillColor, EndAngle;

  MainWindow("Sector demonstration");
  StatusLine("Esc aborts or press a key");
  MaxRadius = MaxY / 10;
  setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
  do {
    FillColor = RandColor();
    setcolor(FillColor);
    setfillstyle(RANDOM_TP(MaxFillStyles), FillColor);
    EndAngle = RANDOM_TP(360);
    sector(RANDOM_TP(MaxX), RANDOM_TP(MaxY), RANDOM_TP(EndAngle), EndAngle,
	   RANDOM_TP(MaxRadius), RANDOM_TP(MaxRadius));
  } while (!kbhit());
  WaitToGo();
}  /* SectorPlay */

#undef MaxFillStyles


#define DelayValue      50   /* milliseconds to delay */


Static void WriteModePlay(void)
{
  /* Demonstrate the SetWriteMode procedure for XOR lines */
  ViewPortType ViewInfo;
  unsigned short Color;
  int Left, Top, Right, Bottom, Step;   /* step for rectangle shrinking */

  MainWindow("SetWriteMode demonstration");
  StatusLine("Esc aborts or press a key");
  getviewsettings(&ViewInfo);
  Left = 0;
  Top = 0;
  Right = ViewInfo.right - ViewInfo.left;
  Bottom = ViewInfo.bottom - ViewInfo.top;
  Step = Bottom / 50;
  setcolor(WHITE);
  line(Left, Top, Right, Bottom);
  line(Left, Bottom, Right, Top);
  setwritemode(XOR_PUT);   /* Set XOR write mode */
  do {
    line(Left, Top, Right, Bottom);   /* Draw XOR lines */
    line(Left, Bottom, Right, Top);
    rectangle(Left, Top, Right, Bottom);   /* Draw XOR rectangle */
    delay((unsigned long)DelayValue);   /* Wait */
    line(Left, Top, Right, Bottom);   /* Erase lines */
    line(Left, Bottom, Right, Top);
    rectangle(Left, Top, Right, Bottom);   /* Erase rectangle */
    if (Left + Step < Right && Top + Step < Bottom) {
      Left += Step;   /* Shrink rectangle */
      Top += Step;
      Right -= Step;
      Bottom -= Step;
    } else {
      Color = RandColor();   /* New color */
      setcolor(Color);
      Left = 0;   /* Original large rectangle */
      Top = 0;
      Right = ViewInfo.right - ViewInfo.left;
      Bottom = ViewInfo.bottom - ViewInfo.top;
    }
  } while (!kbhit());
  setwritemode(COPY_PUT);   /* back to overwrite mode */
  WaitToGo();
}  /* WriteModePlay */

#undef DelayValue


Static void AspectRatioPlay(void)
{
  /* Demonstrate  SetAspectRatio command */
  ViewPortType ViewInfo;
  int CenterX, CenterY;
  unsigned short Radius, Xasp, Yasp;
  int i;
  unsigned short RadiusStep;

  MainWindow("SetAspectRatio demonstration");
  getviewsettings(&ViewInfo);
  CenterX = (ViewInfo.right - ViewInfo.left) / 2;
  CenterY = (ViewInfo.bottom - ViewInfo.top) / 2;
  Radius = (ViewInfo.bottom - ViewInfo.top) / 5 * 3;
  RadiusStep = Radius / 30;
  circle(CenterX, CenterY, Radius);
  getaspectratio(&Xasp, &Yasp);
  for (i = 1; i <= 30; i++) {
    setaspectratio(Xasp, Yasp + i * getmaxx());
	/* Increase Y aspect factor */
    circle(CenterX, CenterY, Radius);
    Radius -= RadiusStep;   /* Shrink radius */
  }
  Radius += RadiusStep * 30;
  for (i = 1; i <= 30; i++) {
    setaspectratio(Xasp + i * getmaxx(), Yasp);
	/* Increase X aspect factor */
    if (Radius > RadiusStep)   /* Shrink radius */
      Radius -= RadiusStep;
    circle(CenterX, CenterY, Radius);
  }
  setaspectratio(Xasp, Yasp);   /* back to original aspect */
  WaitToGo();
}  /* AspectRatioPlay */


Static void TextPlay(void)
{
  /* Demonstrate text justifications and text sizing */
  unsigned short Size, W, H, X, Y;
  ViewPortType ViewInfo;
  Char STR1[256];
  Char STR2[256];

  MainWindow("SetTextJustify / SetUserCharSize demo");
  getviewsettings(&ViewInfo);
  settextstyle(TRIPLEX_FONT, VERT_DIR, 4);
  Y = ViewInfo.bottom - ViewInfo.top - 2;
  settextjustify(CENTER_TEXT, BOTTOM_TEXT);
  outtextxy(textwidth("M") * 2, Y, "Vertical");
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 4);
  settextjustify(LEFT_TEXT, TOP_TEXT);
  outtextxy(textwidth("M") * 2, 2, "Horizontal");
  settextjustify(CENTER_TEXT, CENTER_TEXT);
  X = (ViewInfo.right - ViewInfo.left) / 2;
  Y = textheight("H");
  for (Size = 1; Size <= 4; Size++) {
    settextstyle(TRIPLEX_FONT, HORIZ_DIR, Size);
    H = textheight("M");
    W = textwidth("M");
    Y += H;
    sprintf(STR2, "Size %s", Int2Str(STR1, Size));
    outtextxy(X, Y, STR2);
  }
  Y += H / 2;
  settextjustify(CENTER_TEXT, TOP_TEXT);
  setusercharsize(5, 6, 3, 2);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, USER_CHAR_SIZE);
  outtextxy((ViewInfo.right - ViewInfo.left) / 2, Y, "User defined size!");
  WaitToGo();
}  /* TextPlay */


Static void TextDump(void)
{
  /* Dump the complete character sets to the screen */
  static unsigned short CGASizes[5] = {
    1, 3, 7, 3, 3
  };

  static unsigned short NormSizes[5] = {
    1, 4, 7, 4, 4
  };

  unsigned short Font;
  ViewPortType ViewInfo;
  Char Ch;
  Char STR1[28];
  Char STR2[256];

  for (Font = 0; Font <= 4; Font++) {
    sprintf(STR1, "%s character set", Fonts[Font]);
    MainWindow(STR1);
    getviewsettings(&ViewInfo);
    settextjustify(LEFT_TEXT, TOP_TEXT);
    moveto(2, 3);
    if (Font == DEFAULT_FONT) {   /* with */
      settextstyle(Font, HORIZ_DIR, 1);
      Ch = '\0';
      do {
	sprintf(STR2, "%c", Ch);
	outtext(STR2);
	if (getx() + textwidth("M") > ViewInfo.right - ViewInfo.left)
	  moveto(2, gety() + textheight("M") + 3);
	Ch++;
/* p2c: bgidemo.pas, line 482: Note: Character >= 128 encountered [281] */
      } while ((Ch & 255) < 255);
    } else {
      if (MaxY < 200)
	settextstyle(Font, HORIZ_DIR, CGASizes[Font]);
      else
	settextstyle(Font, HORIZ_DIR, NormSizes[Font]);
      Ch = '!';
      do {
	sprintf(STR2, "%c", Ch);
	outtext(STR2);
	if (getx() + textwidth("M") > ViewInfo.right - ViewInfo.left)
	  moveto(2, gety() + textheight("M") + 3);
	Ch++;
/* p2c: bgidemo.pas, line 496: Note: Character >= 128 encountered [281] */
      } while ((Ch & 255) < 255);
    }
    WaitToGo();
  }  /* for loop */
}  /* TextDump */


#define MaxPoints       15


/* Local variables for LineToPlay: */
struct LOC_LineToPlay {
  unsigned short Xasp, Yasp;
} ;

Local int AdjAsp(int Value, struct LOC_LineToPlay *LINK)
{
  /* Adjust a value for the aspect ratio of the device */
  return ((long)Value * LINK->Xasp / LINK->Yasp);
}  /* AdjAsp */


Static void LineToPlay(void)
{
  /* Demonstrate MoveTo and LineTo commands */
  struct LOC_LineToPlay V;
  PointType Points[MaxPoints + 1];
  ViewPortType ViewInfo;
  int I, J, CenterX;   /* The center point of the circle */
  int CenterY;
  unsigned short Radius, StepAngle;
  double Radians;

  MainWindow("MoveTo, LineTo demonstration");
  getaspectratio(&V.Xasp, &V.Yasp);
  getviewsettings(&ViewInfo);
  CenterX = (ViewInfo.right - ViewInfo.left) / 2;
  CenterY = (ViewInfo.bottom - ViewInfo.top) / 2;
  Radius = CenterY;
  while (CenterY + AdjAsp(Radius, &V) < ViewInfo.bottom - ViewInfo.top - 20)
    Radius++;
  StepAngle = 360 / MaxPoints;
  for (I = 0; I < MaxPoints; I++) {
    Radians = StepAngle * I * M_PI / 180;
    Points[I].x = CenterX + (long)floor(cos(Radians) * Radius + 0.5);
    Points[I].y = CenterY - AdjAsp((long)floor(sin(Radians) * Radius + 0.5), &V);
  }
  circle(CenterX, CenterY, Radius);
  for (I = 0; I < MaxPoints; I++) {
    for (J = I; J < MaxPoints; J++) {
      moveto(Points[I].x, Points[I].y);
      lineto(Points[J].x, Points[J].y);
    }
  }
  WaitToGo();
}  /* LineToPlay */

#undef MaxPoints


#define MaxPoints       12


/* Local variables for LineRelPlay: */
struct LOC_LineRelPlay {
  PointType Poly[MaxPoints];   /* Stores a polygon for filling */
  ViewPortType CurrPort;
} ;

Local void DrawTesseract(struct LOC_LineRelPlay *LINK)
{
  /* Draw a Tesseract on the screen with relative move and
     line drawing commands, also create a polygon for filling */
  static FillPatternType CheckerBoard = {
    0, 0x10, 0x28, 0x44, 0x28, 0x10, 0, 0
  };

  int X, Y, W, H;

  getviewsettings(&LINK->CurrPort);
  W = (LINK->CurrPort.right - LINK->CurrPort.left) / 9;
  H = (LINK->CurrPort.bottom - LINK->CurrPort.top) / 8;
  X = (LINK->CurrPort.right - LINK->CurrPort.left) / 2 - (long)floor(2.5 * W + 0.5);
  Y = (LINK->CurrPort.bottom - LINK->CurrPort.top) / 2 - H * 3;

  /* Border around viewport is outer part of polygon */
  LINK->Poly[0].x = 0;
  LINK->Poly[0].y = 0;
  LINK->Poly[1].x = LINK->CurrPort.right - LINK->CurrPort.left;
  LINK->Poly[1].y = 0;
  LINK->Poly[2].x = LINK->CurrPort.right - LINK->CurrPort.left;
  LINK->Poly[2].y = LINK->CurrPort.bottom - LINK->CurrPort.top;
  LINK->Poly[3].x = 0;
  LINK->Poly[3].y = LINK->CurrPort.bottom - LINK->CurrPort.top;
  LINK->Poly[4].x = 0;
  LINK->Poly[4].y = 0;
  moveto(X, Y);

  /* Grab the whole in the polygon as we draw */
  moverel(0, H);
  LINK->Poly[5].x = getx();
  LINK->Poly[5].y = gety();
  moverel(W, -H);
  LINK->Poly[6].x = getx();
  LINK->Poly[6].y = gety();
  moverel(W * 4, 0);
  LINK->Poly[7].x = getx();
  LINK->Poly[7].y = gety();
  moverel(0, H * 5);
  LINK->Poly[8].x = getx();
  LINK->Poly[8].y = gety();
  moverel(-W, H);
  LINK->Poly[9].x = getx();
  LINK->Poly[9].y = gety();
  moverel(-W * 4, 0);
  LINK->Poly[10].x = getx();
  LINK->Poly[10].y = gety();
  moverel(0, -H * 5);
  LINK->Poly[11].x = getx();
  LINK->Poly[11].y = gety();

  /* Fill the polygon with a user defined fill pattern */
  setfillpattern(CheckerBoard, WHITE);
  fillpoly(12, (void *)LINK->Poly);

  moverel(W, -H);
  linerel(0, H * 5);
  linerel(W * 2, 0);
  linerel(0, -H * 3);
  linerel(W, -H);
  linerel(0, H * 5);
  moverel(0, -H * 5);
  linerel(-W * 2, 0);
  linerel(0, H * 3);
  linerel(-W, H);
  moverel(W, -H);
  linerel(W, 0);
  moverel(0, -H * 2);
  linerel(-W, 0);

  /* Flood fill the center */
  floodfill((LINK->CurrPort.right - LINK->CurrPort.left) / 2,
	    (LINK->CurrPort.bottom - LINK->CurrPort.top) / 2, WHITE);
}  /* DrawTesseract */


Static void LineRelPlay(void)
{
  /* Demonstrate MoveRel and LineRel commands */
  struct LOC_LineRelPlay V;

  MainWindow("LineRel / MoveRel demonstration");
  getviewsettings(&V.CurrPort);
  /* Move the viewport out 1 pixel from each end */
  setviewport(V.CurrPort.left - 1, V.CurrPort.top - 1, V.CurrPort.right + 1,
	      V.CurrPort.bottom + 1, ClipOn);
  DrawTesseract(&V);
  WaitToGo();
}  /* LineRelPlay */

#undef MaxPoints


/* Local variables for PiePlay: */
struct LOC_PiePlay {
  unsigned short Xasp, Yasp;
} ;

Local int AdjAsp_(int Value, struct LOC_PiePlay *LINK)
{
  /* Adjust a value for the aspect ratio of the device */
  return ((long)Value * LINK->Xasp / LINK->Yasp);
}  /* AdjAsp */

Local void GetTextCoords(unsigned short AngleInDegrees, unsigned short Radius,
			 int *X, int *Y, struct LOC_PiePlay *LINK)
{
  /* Get the coordinates of text for pie slice labels */
  double Radians;

  Radians = AngleInDegrees * M_PI / 180;
  *X = (long)floor(cos(Radians) * Radius + 0.5);
  *Y = (long)floor(sin(Radians) * Radius + 0.5);
}  /* GetTextCoords */


Static void PiePlay(void)
{
  /* Demonstrate  PieSlice and GetAspectRatio commands */
  struct LOC_PiePlay V;
  ViewPortType ViewInfo;
  int CenterX, CenterY;
  unsigned short Radius;
  int X, Y;

  MainWindow("PieSlice / GetAspectRatio demonstration");
  getaspectratio(&V.Xasp, &V.Yasp);
  getviewsettings(&ViewInfo);
  CenterX = (ViewInfo.right - ViewInfo.left) / 2;
  CenterY = (ViewInfo.bottom - ViewInfo.top) / 2 + 20;
  Radius = (ViewInfo.bottom - ViewInfo.top) / 3;
  while (AdjAsp_(Radius, &V) <
	 (long)floor((ViewInfo.bottom - ViewInfo.top) / 3.6 + 0.5))
    Radius++;
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 4);
  settextjustify(CENTER_TEXT, TOP_TEXT);
  outtextxy(CenterX, 0, "This is a pie chart!");

  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 3);

  setfillstyle(SOLID_FILL, RandColor());
  pieslice(CenterX + 10, CenterY - AdjAsp_(10, &V), 0, 90, Radius);
  GetTextCoords(45, Radius, &X, &Y, &V);
  settextjustify(LEFT_TEXT, BOTTOM_TEXT);
  outtextxy(CenterX + X + textwidth("H") + 10, CenterY - AdjAsp_(Y + 10, &V),
	    "25 %");

  setfillstyle(HATCH_FILL, RandColor());
  pieslice(CenterX, CenterY, 225, 360, Radius);
  GetTextCoords(293, Radius, &X, &Y, &V);
  settextjustify(LEFT_TEXT, TOP_TEXT);
  outtextxy(CenterX + X + textwidth("H"), CenterY - AdjAsp_(Y, &V), "37.5 %");

  setfillstyle(INTERLEAVE_FILL, RandColor());
  pieslice(CenterX - 10, CenterY, 135, 225, Radius);
  GetTextCoords(180, Radius, &X, &Y, &V);
  settextjustify(RIGHT_TEXT, CENTER_TEXT);
  outtextxy(CenterX + X - textwidth("H") - 10, CenterY - AdjAsp_(Y, &V),
	    "25 %");

  setfillstyle(WIDE_DOT_FILL, RandColor());
  pieslice(CenterX, CenterY, 90, 135, Radius);
  GetTextCoords(112, Radius, &X, &Y, &V);
  settextjustify(RIGHT_TEXT, BOTTOM_TEXT);
  outtextxy(CenterX + X - textwidth("H"), CenterY - AdjAsp_(Y, &V), "12.5 %");

  WaitToGo();
}  /* PiePlay */


#define NumBars         7   /* The number of bars drawn */
#define YTicks          5   /* The number of tick marks on the Y axis */


Static void Bar3DPlay(void)
{
  /* Demonstrate Bar3D command */
  static uchar BarHeight[NumBars] = {
    1, 3, 2, 5, 4, 2, 1
  };

  ViewPortType ViewInfo;
  unsigned short H;
  double XStep, YStep;
  int I, J;
  unsigned short Depth, Color;
  Char STR1[256];

  MainWindow("Bar3D / Rectangle demonstration");
  H = textheight("M") * 3;
  getviewsettings(&ViewInfo);
  settextjustify(CENTER_TEXT, TOP_TEXT);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 4);
  outtextxy(MaxX / 2, 6, "These are 3D bars !");
  settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);
  setviewport(ViewInfo.left + 50, ViewInfo.top + 40, ViewInfo.right - 50,
	      ViewInfo.bottom - 10, ClipOn);
  getviewsettings(&ViewInfo);
  line(H, H, H, ViewInfo.bottom - ViewInfo.top - H);
  line(H, ViewInfo.bottom - ViewInfo.top - H,
       ViewInfo.right - ViewInfo.left - H,
       ViewInfo.bottom - ViewInfo.top - H);
  YStep = (ViewInfo.bottom - ViewInfo.top - H * 2.0) / YTicks;
  XStep = (ViewInfo.right - ViewInfo.left - H * 2.0) / NumBars;
  J = ViewInfo.bottom - ViewInfo.top - H;
  settextjustify(CENTER_TEXT, CENTER_TEXT);

  /* Draw the Y axis and ticks marks */
  for (I = 0; I <= YTicks; I++) {
    line(H / 2, J, H, J);
    outtextxy(0, J, Int2Str(STR1, I));
    J = (long)floor(J - YStep + 0.5);
  }


  Depth = (long)(0.25 * XStep);   /* Calculate depth of bar */

  /* Draw X axis, bars, and tick marks */
  settextjustify(CENTER_TEXT, TOP_TEXT);
  J = H;
  for (I = 1; I <= NumBars + 1; I++) {
    setcolor(WHITE);
    line(J, ViewInfo.bottom - ViewInfo.top - H, J,
	 ViewInfo.bottom - ViewInfo.top - H / 2 - 3);
    outtextxy(J, ViewInfo.bottom - ViewInfo.top - H / 2, Int2Str(STR1, I - 1));
    if (I != NumBars + 1) {
      Color = RandColor();
      setfillstyle(I, Color);
      setcolor(Color);
      bar3d(J, (long)floor(ViewInfo.bottom - ViewInfo.top - H -
			   BarHeight[I - 1] * YStep + 0.5),
	    (long)floor(J + XStep - Depth + 0.5),
	    (long)floor(ViewInfo.bottom - ViewInfo.top - H - 1.0 + 0.5),
	    Depth, TRUE);
      J = (long)floor(J + XStep + 0.5);
    }
  }

  WaitToGo();
}  /* Bar3DPlay */

#undef NumBars
#undef YTicks


#define NumBars         5


Static void BarPlay(void)
{
  /* Demonstrate Bar command */
  static uchar BarHeight[NumBars] = {
    1, 3, 5, 2, 4
  };

  static uchar Styles[NumBars] = {
    1, 3, 10, 5, 9
  };

  ViewPortType ViewInfo;
  unsigned short H;
  double XStep, YStep;
  int I, J;
  unsigned short Color;
  Char STR1[256];

  MainWindow("Bar / Rectangle demonstration");
  H = textheight("M") * 3;
  getviewsettings(&ViewInfo);
  settextjustify(CENTER_TEXT, TOP_TEXT);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 4);
  outtextxy(MaxX / 2, 6, "These are 2D bars !");
  settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);
  setviewport(ViewInfo.left + 50, ViewInfo.top + 30, ViewInfo.right - 50,
	      ViewInfo.bottom - 10, ClipOn);
  getviewsettings(&ViewInfo);
  line(H, H, H, ViewInfo.bottom - ViewInfo.top - H);
  line(H, ViewInfo.bottom - ViewInfo.top - H,
       ViewInfo.right - ViewInfo.left - H,
       ViewInfo.bottom - ViewInfo.top - H);
  YStep = (ViewInfo.bottom - ViewInfo.top - H * 2.0) / NumBars;
  XStep = (ViewInfo.right - ViewInfo.left - H * 2.0) / NumBars;
  J = ViewInfo.bottom - ViewInfo.top - H;
  settextjustify(CENTER_TEXT, CENTER_TEXT);

  /* Draw Y axis with tick marks */
  for (I = 0; I <= NumBars; I++) {
    line(H / 2, J, H, J);
    outtextxy(0, J, Int2Str(STR1, I));
    J = (long)floor(J - YStep + 0.5);
  }

  /* Draw X axis, bars, and tick marks */
  J = H;
  settextjustify(CENTER_TEXT, TOP_TEXT);
  for (I = 1; I <= NumBars + 1; I++) {
    setcolor(WHITE);
    line(J, ViewInfo.bottom - ViewInfo.top - H, J,
	 ViewInfo.bottom - ViewInfo.top - H / 2 - 3);
    outtextxy(J, ViewInfo.bottom - ViewInfo.top - H / 2, Int2Str(STR1, I));
    if (I != NumBars + 1) {
      Color = RandColor();
      setfillstyle(Styles[I - 1], Color);
      setcolor(Color);
      bar(J, (long)floor(ViewInfo.bottom - ViewInfo.top - H -
			 BarHeight[I - 1] * YStep + 0.5),
	  (long)floor(J + XStep + 0.5),
	  ViewInfo.bottom - ViewInfo.top - H - 1);
      rectangle(J, (long)floor(ViewInfo.bottom - ViewInfo.top - H -
			       BarHeight[I - 1] * YStep + 0.5),
		(long)floor(J + XStep + 0.5),
		ViewInfo.bottom - ViewInfo.top - H - 1);
    }
    J = (long)floor(J + XStep + 0.5);
  }

  WaitToGo();
}  /* BarPlay */

#undef NumBars


Static void CirclePlay(void)
{
  /* Draw random circles on the screen */
  unsigned short MaxRadius;

  MainWindow("Circle demonstration");
  StatusLine("Esc aborts or press a key");
  MaxRadius = MaxY / 10;
  setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
  do {
    setcolor(RandColor());
    circle(RANDOM_TP(MaxX), RANDOM_TP(MaxY), RANDOM_TP(MaxRadius));
  } while (!kbhit());
  WaitToGo();
}  /* CirclePlay */


Static void RandBarPlay(void)
{
  /* Draw random bars on the screen */
  int MaxWidth, MaxHeight;
  ViewPortType ViewInfo;
  unsigned short Color;

  MainWindow("Random Bars");
  StatusLine("Esc aborts or press a key");
  getviewsettings(&ViewInfo);
  MaxWidth = ViewInfo.right - ViewInfo.left;
  MaxHeight = ViewInfo.bottom - ViewInfo.top;
  do {
    Color = RandColor();
    setcolor(Color);
    setfillstyle(RANDOM_TP(CLOSE_DOT_FILL) + 1, Color);
    bar3d(RANDOM_TP(MaxWidth), RANDOM_TP(MaxHeight), RANDOM_TP(MaxWidth),
	  RANDOM_TP(MaxHeight), 0, FALSE);
  } while (!kbhit());
  WaitToGo();
}  /* RandBarPlay */


Static void ArcPlay(void)
{
  /* Draw random arcs on the screen */
  unsigned short MaxRadius, EndAngle;
  ArcCoordsType ArcInfo;

  MainWindow("Arc / GetArcCoords demonstration");
  StatusLine("Esc aborts or press a key");
  MaxRadius = MaxY / 10;
  do {
    setcolor(RandColor());
    EndAngle = RANDOM_TP(360);
    setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
    arc(RANDOM_TP(MaxX), RANDOM_TP(MaxY), RANDOM_TP(EndAngle), EndAngle,
	RANDOM_TP(MaxRadius));
    getarccoords(&ArcInfo);
    line(ArcInfo.x, ArcInfo.y, ArcInfo.xstart, ArcInfo.ystart);
    line(ArcInfo.x, ArcInfo.y, ArcInfo.xend, ArcInfo.yend);
  } while (!kbhit());
  WaitToGo();
}  /* ArcPlay */


#define Seed            1962   /* A seed for the random number generator */
#define NumPts          2000   /* The number of pixels plotted */

#define Esc             '\033'


Static void PutPixelPlay(void)
{
  /* Demonstrate the PutPixel and GetPixel commands */
  unsigned short I, X, Y, Color;
  int XMax, YMax;
  ViewPortType ViewInfo;

  MainWindow("PutPixel / GetPixel demonstration");
  StatusLine("Esc aborts or press a key...");

  getviewsettings(&ViewInfo);
  XMax = ViewInfo.right - ViewInfo.left - 1;
  YMax = ViewInfo.bottom - ViewInfo.top - 1;

  while (!kbhit()) {
    /* Plot random pixels */
    RANDSEED_TP = Seed;
    I = 0;
    while (!kbhit() && I < NumPts) {
      I++;
      X = RANDOM_TP(XMax) + 1;
      Y = RANDOM_TP(YMax) + 1;
      putpixel(X, Y, RandColor());
    }

    /* Erase pixels */
    RANDSEED_TP = Seed;
    I = 0;
    while (!kbhit() && I < NumPts) {
      I++;
      X = RANDOM_TP(XMax) + 1;
      Y = RANDOM_TP(YMax) + 1;
      Color = getpixel(X, Y);
      if (Color == RandColor())
	putpixel(X, Y, 0);
    }
    if (!kbhit())
      delay((unsigned long)500);
  }
  WaitToGo();
}  /* PutPixelPlay */

#undef Seed
#undef NumPts
#undef Esc


#define r               20
#define StartX          100
#define StartY          50


/* Local variables for PutImagePlay: */
struct LOC_PutImagePlay {

  ViewPortType CurPort;
} ;

Local void MoveSaucer(int *X, int *Y, int Width, int Height,
		      struct LOC_PutImagePlay *LINK)
{
  int Step;

  Step = RANDOM_TP(r * 2);
  if (Step & 1)
    Step = -Step;
  *X += Step;
  Step = RANDOM_TP(r);
  if (Step & 1)
    Step = -Step;
  *Y += Step;

  /* Make saucer bounce off viewport walls */
  if (LINK->CurPort.left + *X + Width - 1 > LINK->CurPort.right)
    *X = LINK->CurPort.right - LINK->CurPort.left - Width + 1;
  else {
    if (*X < 0)
      *X = 0;
  }
  if (LINK->CurPort.top + *Y + Height - 1 > LINK->CurPort.bottom)
    *Y = LINK->CurPort.bottom - LINK->CurPort.top - Height + 1;
  else {
    if (*Y < 0)
      *Y = 0;
  }
}  /* MoveSaucer */


Static void PutImagePlay(void)
{
  /* Demonstrate the GetImage and PutImage commands */
  struct LOC_PutImagePlay V;

  unsigned short Pausetime;
  void *Saucer;
  int X, Y;
  unsigned short ulx, uly, lrx, lry, Size, I;

  cleardevice();
  FullPort();

  /* PaintScreen */
  cleardevice();
  MainWindow("GetImage / PutImage Demonstration");
  StatusLine("Esc aborts or press a key...");
  getviewsettings(&V.CurPort);

  /* DrawSaucer */
  ellipse(StartX, StartY, 0, 360, r, r / 3 + 2);
  ellipse(StartX, StartY - 4, 190, 357, r, r / 3);
  line(StartX + 7, StartY - 6, StartX + 10, StartY - 12);
  circle(StartX + 10, StartY - 12, 2);
  line(StartX - 7, StartY - 6, StartX - 10, StartY - 12);
  circle(StartX - 10, StartY - 12, 2);
  setfillstyle(SOLID_FILL, WHITE);
  floodfill(StartX + 1, StartY + 4, getcolor());

  /* ReadSaucerImage */
  ulx = StartX - r - 1;
  uly = StartY - 14;
  lrx = StartX + r + 1;
  lry = StartY + r / 3 + 3;

  Size = imagesize(ulx, uly, lrx, lry);
  Saucer = Malloc(Size);
  getimage(ulx, uly, lrx, lry, Saucer);
  putimage(ulx, uly, Saucer, XOR_PUT);   /* erase image */

  /* Plot some "stars" */
  for (I = 1; I <= 1000; I++)
    putpixel(RANDOM_TP(MaxX), RANDOM_TP(MaxY), RandColor());
  X = MaxX / 2;
  Y = MaxY / 2;
  Pausetime = 70;

  /* Move the saucer around */
  do {
    putimage(X, Y, Saucer, XOR_PUT);   /* draw image */
    delay((unsigned long)Pausetime);
    putimage(X, Y, Saucer, XOR_PUT);   /* erase image */
    MoveSaucer(&X, &Y, lrx - ulx + 1, lry - uly + 1, &V);   /* width/height */
  } while (!kbhit());
/* p2c: bgidemo.pas, line 1041:
 * Warning: Too many arguments for freemem [299] */
  Free(Saucer);
  WaitToGo();
}  /* PutImagePlay */

#undef r
#undef StartX
#undef StartY


#define MaxPts          5


typedef PointType PolygonType[MaxPts];


Static void PolyPlay(void)
{
  /* Draw random polygons with random fill styles on the screen */
  PolygonType Poly;
  unsigned short I, Color;
  PointType *WITH;

  MainWindow("FillPoly demonstration");
  StatusLine("Esc aborts or press a key...");
  do {
    Color = RandColor();
    setfillstyle(RANDOM_TP(11) + 1, Color);
    setcolor(Color);
    for (I = 0; I < MaxPts; I++) {
      WITH = &Poly[I];
      WITH->x = RANDOM_TP(MaxX);
      WITH->y = RANDOM_TP(MaxY);
    }
    fillpoly(MaxPts, (void *)Poly);
  } while (!kbhit());
  WaitToGo();
}  /* PolyPlay */

#undef MaxPts


/* Local variables for FillStylePlay: */
struct LOC_FillStylePlay {
  unsigned short Style, Width, Height;
  ViewPortType ViewInfo;
} ;

Local void DrawBox(unsigned short X, unsigned short Y,
		   struct LOC_FillStylePlay *LINK)
{
  Char STR1[256];

  setfillstyle(LINK->Style, WHITE);
  bar(X, Y, X + LINK->Width, Y + LINK->Height);
  rectangle(X, Y, X + LINK->Width, Y + LINK->Height);
  outtextxy(X + LINK->Width / 2, Y + LINK->Height + 4,
	    Int2Str(STR1, LINK->Style));
  LINK->Style++;
}  /* DrawBox */


Static void FillStylePlay(void)
{
  /* Display all of the predefined fill styles available */
  struct LOC_FillStylePlay V;
  unsigned short X, Y, I, J;

  MainWindow("Pre-defined fill styles");
  getviewsettings(&V.ViewInfo);
  V.Width = (V.ViewInfo.right + 1) / 13 * 2;
  V.Height = (V.ViewInfo.bottom - 10) / 10 * 2;
  X = V.Width / 2;
  Y = V.Height / 2;
  V.Style = 0;
  for (J = 1; J <= 3; J++) {
    for (I = 1; I <= 4; I++) {
      DrawBox(X, Y, &V);
      X += V.Width / 2 * 3;
    }
    X = V.Width / 2;
    Y += V.Height / 2 * 3;
  }
  settextjustify(LEFT_TEXT, TOP_TEXT);
  WaitToGo();
}  /* FillStylePlay */


/* Local variables for FillPatternPlay: */
struct LOC_FillPatternPlay {
  FillPatternType Patterns[12];
  unsigned short Style, Width, Height;
  ViewPortType ViewInfo;
} ;

Local void DrawBox_(unsigned short X, unsigned short Y,
		    struct LOC_FillPatternPlay *LINK)
{
  setfillpattern(LINK->Patterns[LINK->Style], WHITE);
  bar(X, Y, X + LINK->Width, Y + LINK->Height);
  rectangle(X, Y, X + LINK->Width, Y + LINK->Height);
  LINK->Style++;
}  /* DrawBox */


Static void FillPatternPlay(void)
{
  /* Display some user defined fill patterns */
  struct LOC_FillPatternPlay V;

  static FillPatternType Patterns[12] = {
    { 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55 },
    { 0x33, 0x33, 0xcc, 0xcc, 0x33, 0x33, 0xcc, 0xcc },
    { 0xf0, 0xf0, 0xf0, 0xf0, 0xf, 0xf, 0xf, 0xf },
    { 0, 0x10, 0x28, 0x44, 0x28, 0x10, 0, 0 },
    { 0, 0x70, 0x20, 0x27, 0x25, 0x27, 0x4, 0x4 },
    { 0, 0, 0, 0x18, 0x18, 0, 0, 0 },
    { 0, 0, 0x3c, 0x3c, 0x3c, 0x3c, 0, 0 },
    { 0, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 0 },
    { 0, 0, 0x22, 0x8, 0, 0x22, 0x1c, 0 },
    { 0xff, 0x7e, 0x3c, 0x18, 0x18, 0x3c, 0x7e, 0xff },
    { 0, 0x10, 0x10, 0x7c, 0x10, 0x10, 0, 0 },
    { 0, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0 }
  };

  unsigned short X, Y, I, J;

  memcpy(V.Patterns, Patterns, sizeof(FillPatternType[12]));
  MainWindow("User defined fill styles");
  getviewsettings(&V.ViewInfo);
  V.Width = (V.ViewInfo.right + 1) / 13 * 2;
  V.Height = (V.ViewInfo.bottom - 10) / 10 * 2;
  X = V.Width / 2;
  Y = V.Height / 2;
  V.Style = 0;
  for (J = 1; J <= 3; J++) {
    for (I = 1; I <= 4; I++) {
      DrawBox_(X, Y, &V);
      X += V.Width / 2 * 3;
    }
    X = V.Width / 2;
    Y += V.Height / 2 * 3;
  }
  settextjustify(LEFT_TEXT, TOP_TEXT);
  WaitToGo();
}  /* FillPatternPlay */


/* Local variables for ColorPlay: */
struct LOC_ColorPlay {
  unsigned short Color, Width, Height;
  ViewPortType ViewInfo;
} ;

Local void DrawBox__(unsigned short X, unsigned short Y,
		     struct LOC_ColorPlay *LINK)
{
  Char STR1[256];

  setfillstyle(SOLID_FILL, LINK->Color);
  setcolor(LINK->Color);
  bar(X, Y, X + LINK->Width, Y + LINK->Height);
  rectangle(X, Y, X + LINK->Width, Y + LINK->Height);
  LINK->Color = getcolor();
  if (LINK->Color == 0) {
    setcolor(WHITE);
    rectangle(X, Y, X + LINK->Width, Y + LINK->Height);
  }
  outtextxy(X + LINK->Width / 2, Y + LINK->Height + 4,
	    Int2Str(STR1, LINK->Color));
  LINK->Color = (LINK->Color + 1) % (MaxColor + 1);
}  /* DrawBox */


Static void ColorPlay(void)
{
  /* Display all of the colors available for the current driver and mode */
  struct LOC_ColorPlay V;
  unsigned short X, Y, I, J;

  MainWindow("Color demonstration");
  V.Color = 1;
  getviewsettings(&V.ViewInfo);
  V.Width = (V.ViewInfo.right + 1) / 16 * 2;
  V.Height = (V.ViewInfo.bottom - 10) / 10 * 2;
  X = V.Width / 2;
  Y = V.Height / 2;
  for (J = 1; J <= 3; J++) {
    for (I = 1; I <= 5; I++) {
      DrawBox__(X, Y, &V);
      X += V.Width / 2 * 3;
    }
    X = V.Width / 2;
    Y += V.Height / 2 * 3;
  }
  WaitToGo();
}  /* ColorPlay */


#define XBars           15
#define YBars           10


Static void PalettePlay(void)
{
  /* Demonstrate the use of the SetPalette command */
  unsigned short I, J, X, Y, Cols, Color;
  ViewPortType ViewInfo;
  unsigned short Width, Height;
  PaletteType OldPal;

  getpalette(&OldPal);
  MainWindow("Palette demonstration");
  StatusLine("Press any key...");
  getviewsettings(&ViewInfo);
  Width = (ViewInfo.right - ViewInfo.left) / XBars;
  Height = (ViewInfo.bottom - ViewInfo.top) / YBars;
  X = 0;
  Y = 0;
  Color = 1;
  Cols = 16;
  if (MaxColor < Cols)
    Cols = MaxColor;
  for (J = 1; J <= YBars; J++) {
    for (I = 1; I <= XBars; I++) {
      setfillstyle(SOLID_FILL, Color);
      bar(X, Y, X + Width, Y + Height);
      X += Width + 1;
      Color++;
      Color = Color % (Cols - 2) + 1;
    }
    X = 0;
    Y += Height + 1;
  }
  do {
    setpalette(RANDOM_TP(Cols - 2) + 1, RANDOM_TP(64));
  } while (!kbhit());
  setallpalette((void *)(&OldPal));
  WaitToGo();
}  /* PalettePlay */

#undef XBars
#undef YBars


Static void CrtModePlay(void)
{
  /* Demonstrate the use of RestoreCrtMode and SetGraphMode */
  ViewPortType ViewInfo;
  Char Ch;

  MainWindow("SetGraphMode / RestoreCrtMode demo");
  getviewsettings(&ViewInfo);
  settextjustify(CENTER_TEXT, CENTER_TEXT);
  outtextxy((ViewInfo.right - ViewInfo.left) / 2,
	    (ViewInfo.bottom - ViewInfo.top) / 2,
	    "Now you are in graphics mode");
  StatusLine("Press any key for text mode...");
  do {
  } while (!kbhit());
  Ch = getch();
  if (Ch == '\0')   /* trap function keys */
    Ch = getch();
  restorecrtmode();
  printf("Now you are in text mode.\n");
  printf("Press any key to go back to graphics...");
  do {
  } while (!kbhit());
  Ch = getch();
  if (Ch == '\0')   /* trap function keys */
    Ch = getch();
  setgraphmode(getgraphmode());
  MainWindow("SetGraphMode / RestoreCrtMode demo");
  settextjustify(CENTER_TEXT, CENTER_TEXT);
  outtextxy((ViewInfo.right - ViewInfo.left) / 2,
	    (ViewInfo.bottom - ViewInfo.top) / 2, "Back in graphics mode...");
  WaitToGo();
}  /* CrtModePlay */


Static void LineStylePlay(void)
{
  /* Demonstrate the predefined line styles available */
  unsigned short Style, Step, X, Y;
  ViewPortType ViewInfo;
  Char STR1[256];

  cleardevice();
  DefaultColors();
  MainWindow("Pre-defined line styles");
  getviewsettings(&ViewInfo);
  X = 35;
  Y = 10;
  Step = (ViewInfo.right - ViewInfo.left) / 11;
  settextjustify(LEFT_TEXT, TOP_TEXT);
  outtextxy(X, Y, "NormWidth");
  settextjustify(CENTER_TEXT, TOP_TEXT);
  for (Style = 0; Style <= 3; Style++) {
    setlinestyle(Style, 0, NORM_WIDTH);
    line(X, Y + 20, X, ViewInfo.bottom - 40);
    outtextxy(X, ViewInfo.bottom - 30, Int2Str(STR1, Style));
    X += Step;
  }
  X += Step * 2;
  settextjustify(LEFT_TEXT, TOP_TEXT);
  outtextxy(X, Y, "ThickWidth");
  settextjustify(CENTER_TEXT, TOP_TEXT);
  for (Style = 0; Style <= 3; Style++) {
    setlinestyle(Style, 0, THICK_WIDTH);
    line(X, Y + 20, X, ViewInfo.bottom - 40);
    outtextxy(X, ViewInfo.bottom - 30, Int2Str(STR1, Style));
    X += Step;
  }
  settextjustify(LEFT_TEXT, TOP_TEXT);
  WaitToGo();
}  /* LineStylePlay */


Static void UserLineStylePlay(void)
{
  /* Demonstrate user defined line styles */
  unsigned short Style, X, Y, I;
  ViewPortType ViewInfo;

  MainWindow("User defined line styles");
  getviewsettings(&ViewInfo);
  X = 4;
  Y = 10;
  Style = 0;
  I = 0;
  while (X < ViewInfo.right - 4) {
    Style |= 1 << (I & 15);
    setlinestyle(USERBIT_LINE, Style, NORM_WIDTH);
    line(X, Y, X, ViewInfo.bottom - ViewInfo.top - Y);
    X += 5;
    I++;
    if (Style == 65535L) {
      I = 0;
      Style = 0;
    }
  }
  WaitToGo();
}  /* UserLineStylePlay */


Static void SayGoodbye(void)
{
  /* Say goodbye and then exit the program */
  ViewPortType ViewInfo;

  MainWindow("");
  getviewsettings(&ViewInfo);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 4);
  settextjustify(CENTER_TEXT, CENTER_TEXT);
  outtextxy((ViewInfo.right - ViewInfo.left) / 2,
	    (ViewInfo.bottom - ViewInfo.top) / 2, "That's all folks!");
  StatusLine("Press any key to quit...");
  do {
  } while (!kbhit());
}  /* SayGoodbye */


main(int argc, Char *argv[])
{  /* program body */
  PASCAL_MAIN(argc, argv);
  Initialize();
  ReportStatus();

  AspectRatioPlay();
  FillEllipsePlay();
  SectorPlay();
  WriteModePlay();

  ColorPlay();
  /* PalettePlay only intended to work on these drivers: */
  if (GraphDriver == EGA || GraphDriver == EGA64 || GraphDriver == VGA)
    PalettePlay();

  PutPixelPlay();
  PutImagePlay();
  RandBarPlay();
  BarPlay();
  Bar3DPlay();
  ArcPlay();
  CirclePlay();
  PiePlay();
  LineToPlay();
  LineRelPlay();
  LineStylePlay();
  UserLineStylePlay();
  TextDump();
  TextPlay();
  CrtModePlay();
  FillStylePlay();
  FillPatternPlay();
  PolyPlay();
  SayGoodbye();
  closegraph();
  exit(EXIT_SUCCESS);
}



/* End. */
